require 'do-list/version'
require 'do-list/auth_token'
require 'do-list/push_entry'
require 'do-list/url_parser'

module DoList
  class Integration
    def initialize(params = {})
      @account_id = params[:account_id]
      @api_key = params[:api_key]
      @auth_url = 'http://api.dolist.net/v2/AuthenticationService.svc/soap1.1'
      @push_url = 'http://api.dolist.net/V2/ContactManagementService.svc/soap1.1'
      @auth_token = DoList::AuthToken.new(account_id: @account_id,
                                          api_key: @api_key,
                                          url: @auth_url,
                                          headers: create_headers('auth'))
    end

    def auth_key
      @auth_token.auth_key
    end

    def push_entry(fields)
      entry = DoList::PushEntry.new(auth_token: @auth_token.auth_key,
                                    account_id: @account_id,
                                    url: @push_url,
                                    headers: create_headers('push'))
      entry.push_entry(fields)
    end

    private

    def create_headers(type)
      path = 'AuthenticationService/GetAuthenticationToken' if type == 'auth'
      path = 'ContactManagementService/SaveContact' if type == 'push'
      {
        'Content-Type' => 'text/xml; charset=utf-8',
        'SOAPAction' => "http://api.dolist.net/v2/#{path}"
      }
    end
  end
end

require 'date'
require 'net/https'
require 'uri'
require 'nokogiri'
